<?php
// https://understanddrupal.com/articles/how-debug-drupal-migrations-part-2

namespace Drupal\popsu_migrate\Plugin\migrate\source;

use Drupal\migrate\Row;
use Drush\Drush;
use Drupal\paragraphs\Entity\Paragraph;

// use Drupal\popsu_migrate\Plugin\migrate\source\D7Programme;

/**
 * Drupal 7 node source from database.
 *
 * @MigrateSource(
 *   id = "d7_node_programme",
 *   source_module = "node"
 * )
 */
class D7NodeProgramme extends D7Programme {

  /**
   * {@inheritdoc}
   */
  public function query() {
    $query = parent::query($row);

    // field_popsu_special_typetaxo 31 (Acueile)
    $query->leftJoin('field_revision_field_popsu_special_typetaxo', 'ff', 'ff.revision_id = n.vid');
    $query->fields('ff', [
      'field_popsu_special_typetaxo_tid'
    ]);
    $query->condition('ff.field_popsu_special_typetaxo_tid', 31);

    return $query;
  }

  /**
   * {@inheritdoc}
   */
  public function prepareRow(Row $row) {
    $nid = $row->getSourceProperty('nid');
    $vid = $row->getSourceProperty('vid');
    $type = $row->getSourceProperty('type');

    // Get Field API field values.
    foreach ($this->getFields('node', $type) as $field_name => $field) {
      // Ensure we're using the right language if the entity and the field are
      // translatable.
      // $field_language = $entity_translatable && $field['translatable'] ? $language : NULL;
      $row->setSourceProperty($field_name, $this->getFieldValues('node', $field_name, $nid, $vid, NULL));
    }

    // title from taxo term
    $field_popsu_special_popsu_tid = $row->getSourceProperty('field_popsu_special_popsu')[0]['tid'];
    $q = $this->select('taxonomy_term_data', 'ttd')
      ->fields('ttd', ['tid','name'])
      ->condition('tid', $field_popsu_special_popsu_tid);
    $term = $q->execute()->fetchAll();
    // Drush::output()->writeln(dump($term));
    $row->setSourceProperty('term_title', $term[0]['name']);

    // text intro
    $body = $row->getSourceProperty('field_popsu_special_body');
    $row->setSourceProperty('body', $body[0]['value']);
    $row->setSourceProperty('body_summary', $body[0]['summary']);

    // associateds
    $associateds = $this->getAssociatedSpecial($field_popsu_special_popsu_tid);
    Drush::output()->writeln(dump($associateds));

    // texts (migration_lookup)
    $text = $row->getSourceProperty('field_popsu_special_text');
    $text[0]['format'] = 'wysiwyg';
    $p = Paragraph::create(['type' => 'textes']);
    $p->set('field_texte', $text)
      ->set('field_titre', $row->getSourceProperty('title'))
      ->isNew();
    $p->save();

    $text_paragraphe = array(
      array(
        'target_id' => $p->id(),
        'target_revision_id' => $p->getRevisionId(),
      ),
    );
    $row->setSourceProperty('text_paragraphe', $text_paragraphe);

    // Sous partie

    // fichiers
    // files from current node
    $fichiers = array();
    foreach ($row->getSourceProperty('field_popsu_special_insert') as $value) {
      $fichiers[] = $value;
    }
    foreach ($row->getSourceProperty('field_popsu_special_insert_intro') as $value) {
      $fichiers[] = $value;
    }
    // files from associated nodes (colloques, publications, partenaires)
    foreach ( ['colloques', 'publications'] as $name) {
      foreach ($associateds[$name] as $key => $asso) {
        // tried with array merge but it failed
        // and apparently foreach is faster than array_merge
        foreach ($asso['field_popsu_special_insert'] as $value) {
          $fichiers[] = $value;
        }
        foreach ($asso['field_popsu_special_insert_intro'] as $value) {
          $fichiers[] = $value;
        }
      }
    }
    $row->setSourceProperty('fichiers', $fichiers);

    return parent::prepareRow($row);
  }

}
